/*
 * File:   DBManager.h
 * Author: mdm
 *
 * Created on October 24, 2013, 2:57 PM
 */

#ifndef DBMANAGER_H
#define	DBMANAGER_H

#include <Configuration.h>

#include <tango.h>

#include <ctime>

#include <boost/tuple/tuple.hpp>
#include <boost/optional/optional.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/thread/mutex.hpp>

#include <soci/soci.h>
#include <soci/error.h>
#include <soci/row.h>
#include <soci/rowset.h>
#include <soci/boost-tuple.h>
#include <soci/boost-optional.h>
#include <soci/session.h>

namespace DataImporter_ns
{

class DBManager : public Tango::LogAdapter
{
public:
//------------------------------------------------------------------------------
//  [Public] Shared pointer typedef
//------------------------------------------------------------------------------
    typedef boost::shared_ptr<DBManager> SP;

protected:
//------------------------------------------------------------------------------
//  [Protected] Constructor destructor deleter
//------------------------------------------------------------------------------
    DBManager(Tango::DeviceImpl*, Configuration::SP);

    virtual ~DBManager();

    class Deleter;
    friend Deleter;
    class Deleter
    {
        public:
            void operator()(DBManager* d) { delete d; }
    };

public:
//------------------------------------------------------------------------------
//  [Public] Class creation method
//------------------------------------------------------------------------------
    static DBManager::SP create(Tango::DeviceImpl*, Configuration::SP);

//------------------------------------------------------------------------------
//  [Public] Connection handling methods
//------------------------------------------------------------------------------
    virtual void connect() throw(soci::soci_error);

    virtual void disconnect();

//------------------------------------------------------------------------------
//  [Public] Timestamp methods
//------------------------------------------------------------------------------
    virtual boost::posix_time::ptime retrieveLastTimestamp()
        throw(std::runtime_error, std::out_of_range);

    virtual void persistLastTimestamp(boost::posix_time::ptime)
        throw(std::runtime_error);

//------------------------------------------------------------------------------
//  [Public] New file method
//------------------------------------------------------------------------------
    typedef boost::tuple< boost::optional<int>, boost::optional<std::string>,
        boost::optional<std::tm> > FileRow;

    typedef soci::rowset< FileRow > FileRowset;

    typedef boost::shared_ptr< FileRowset > FileRowsetSP;

    virtual FileRowsetSP retrieveNewFile(std::string, std::string,
        boost::posix_time::ptime) throw(soci::soci_error);

protected:
//------------------------------------------------------------------------------
//  [Protected] Class variables
//------------------------------------------------------------------------------
    //Device name
    std::string m_deviceName;

    //Configuration shared pointer
    Configuration::SP m_configuration_sp;

    //Metadata database connection mutex
    boost::mutex m_sessionMutex;

    //Metadata database connection scoped pointer
    boost::scoped_ptr<soci::session> m_session_sp;

    //Tango database connection mutex
    boost::mutex m_tangoDBMutex;

    //Tango database connection scoped pointer
    boost::scoped_ptr<Tango::Database> m_tangoDB_sp;
};

}   //End of namespace

#endif	/* DBMANAGER_H */
